<?php

declare(strict_types=1);

namespace Drupal\calculation_fields_example\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provide a simple form just as an example of the calculation fields.
 *
 * @see \Drupal\calculation_fields_example\Controller\MultipleFormsAtSamePageController
 */
final class QuantityField1 extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'calculation_fields_example_quantity_field1';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['quantity'] = [
      '#type' => 'number',
      '#title' => $this->t('Quantity'),
      '#required' => TRUE,
      '#default_value' => 1,
    ];

    $form['product_price'] = [
      '#type' => 'number',
      '#title' => $this->t('Product price'),
      '#required' => TRUE,
      '#default_value' => 10,
      '#readonly' => TRUE,
    ];

    $form['total'] = [
      '#type' => 'form_calculation_element',
      '#title' => $this->t('Total'),
      '#evaluation_fields' => '(:quantity * :product_price)',
      '#evaluation_decimals' => 1,
    ];

    $form['total_with_discount'] = [
      '#type' => 'form_calculation_markup',
      '#title' => $this->t('Total - discount of 10%'),
      '#placeholder' => $this->t('Add product to cart to see the result'),
      '#evaluation_fields' => '<h3>Total with discount: <span>{{ (:quantity * :product_price) * 0.90 }}</span></h3>',
      '#evaluation_decimals' => 1,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      'submit' => [
        '#type' => 'submit',
        '#value' => $this->t('Add'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {}

}
