<?php

namespace Drupal\Tests\calculation_fields\Functional;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Test the entire form feature with the calculation elements.
 */
class CalculationFieldsFormTest extends WebDriverTestBase {

  /**
   * Modules to load.
   *
   * @var string[]
   */
  protected static $modules = [
    'user',
    'calculation_fields',
    'calculation_fields_example',
  ];

  /**
   * Default theme to use on the tests.
   *
   * @var string
   */
  protected $defaultTheme = 'olivero';

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();
    $admin_user = $this->drupalCreateUser([], 'admin-test', TRUE);
    $this->drupalLogin($admin_user);
  }

  /**
   * Test calculation field elements in the form built.
   */
  public function testCalculationFieldFormExample() {
    $this->drupalGet('/calculation-fields-example/form');
    $session = $this->getSession();
    $page = $session->getPage();

    $markupElement = $page->find('css', 'div[data-drupal-selector="calculation_markup_calculation-fields-example_first_value_second_value_third_value_multiple"]');

    $this->assertTrue($markupElement->find('css', '.js-math-placeholder')->isVisible());
    $this->assertFalse($markupElement->find('css', '.js-math-result')->isVisible());
    $this->assertFalse($markupElement->find('css', '.js-math-result-base')->isVisible());
    // Set values for first and second field.
    // For the third field has a default value and the calculation should work.
    $page->findField('first_value')->setValue("10");
    $page->findField('second_value')->setValue("20");
    // Result still not visible.
    $this->assertFalse($markupElement->find('css', '.js-math-result')->isVisible());
    $page->findField('multiple')->setValue("2");
    $this->assertFalse($markupElement->find('css', '.js-math-placeholder')->isVisible());
    $page->findField('multiple')->setValue("");
    $this->assertTrue($markupElement->find('css', '.js-math-placeholder')->isVisible());
    $page->findField('multiple')->setValue("2");
    $this->assertFalse($markupElement->find('css', '.js-math-placeholder')->isVisible());

    $result = $page->findField('result')->getValue();
    $this->assertEquals($result, 64);

    $page->findField('multiple')->setValue("3");
    $result = $page->findField('result')->getValue();
    $this->assertEquals($result, 96);

    $page->findField('first_value')->setValue("20");
    $page->findField('third_value')->setValue("0");
    $result = $page->findField('result')->getValue();
    $this->assertEquals($result, 120);

    // Placeholder must be hidden after calculation.
    $this->assertFalse($markupElement->find('css', '.js-math-placeholder')->isVisible());
    $this->assertTrue($markupElement->find('css', '.js-math-result')->isVisible());
    $this->assertFalse($markupElement->find('css', '.js-math-result-base')->isVisible());
    $this->assertEquals("<h3>This is my test result divided by 2: <span>R$60.00</span></h3>", $markupElement->find('css', '.js-math-result')->getHtml());
  }

  /**
   * Test with multiple forms on the same page that use the same field name.
   */
  public function testCalculationWithMultipleForms() {
    $this->drupalGet('/calculation-fields-example/multiple-forms-at-same-page');
    $session = $this->getSession();
    $page = $session->getPage();

    $form1 = $page->findById('calculation-fields-example-quantity-field1');
    $form2 = $page->findById('calculation-fields-example-quantity-field2');

    $markupElement = $page->find('css', 'div[data-drupal-selector="calculation_markup_calculation-fields-example-quantity-field1_quantity_product_price"]');
    $form1->findField('quantity')->setValue("10");
    $this->assertEquals($form1->findField('total')->getValue(), "100.0");
    $this->assertEquals("<h3>Total with discount: <span>90.0</span></h3>", $markupElement->find('css', '.js-math-result')->getHtml());

    $markupElement = $page->find('css', 'div[data-drupal-selector="calculation_markup_calculation-fields-example-quantity-field2_quantity_product_price"]');
    $this->assertTrue($markupElement->find('css', '.js-math-placeholder')->isVisible());
    $form2->findField('quantity')->setValue("1");
    $this->assertEquals($form2->findField('total')->getValue(), "5.0");
    $this->assertEquals("<h3>Total with discount: <span>4.8</span></h3>", $markupElement->find('css', '.js-math-result')->getHtml());
    $this->assertFalse($markupElement->find('css', '.js-math-placeholder')->isVisible());
  }

  /**
   * Test the currency settings.
   */
  public function testCurrencySettings() {
    $this->drupalGet('/calculation-fields-mask-example/form');
    $session = $this->getSession();
    $page = $session->getPage();

    $page->findField('first_value')->setValue("2");
    $page->findField('second_value')->setValue("3");
    $page->findField('third_value')->setValue("5");
    $page->findField('multiple')->setValue("2");

    $result = $page->findField('result')->getValue();
    $this->assertEquals($result, "€20.00");
  }

}
